/**************************************************************************//**
*   @file   ADF4106.c
*   @brief  Implementation of ADF4106 Driver for Nios II processor.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#define ADF4106_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include <altera_avalon_pio_regs.h>	// PIO access macros
#include <altera_avalon_spi.h>		// SPI
#include <unistd.h>					// usleep();
#include <system.h>
#include "adf4106.h"

#if(USE_ADF4106 == 1)

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
unsigned char   txBuffer[4]     = {0,}; // Tx SPI Buffer.
unsigned char   rxBuffer[4]     = {0,}; // Rx SPI Buffer.

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
volatile char* 	adf4106_LDPbtn		= &ADF4106_Data[0].chData;
volatile char* 	adf4106_ABP2btn		= &ADF4106_Data[1].chData;
volatile char* 	adf4106_ABP1btn		= &ADF4106_Data[2].chData;
volatile short* adf4106_RSlider		= &ADF4106_Data[3].siData;
volatile char* 	adf4106_CPGainbtn	= &ADF4106_Data[4].chData;
volatile short* adf4106_BSlider		= &ADF4106_Data[5].siData;
volatile short* adf4106_ASlider		= &ADF4106_Data[6].siData;
volatile char* 	adf4106_PrescVal	= &ADF4106_Data[7].chData;
volatile char* 	adf4106_PD2			= &ADF4106_Data[8].chData;
volatile char* 	adf4106_CurrSet2	= &ADF4106_Data[9].chData;
volatile char* 	adf4106_CurrSet1	= &ADF4106_Data[10].chData;
volatile char* 	adf4106_TimerCtrl	= &ADF4106_Data[11].chData;
volatile char* 	adf4106_FastLock	= &ADF4106_Data[12].chData;
volatile char* 	adf4106_FastEn		= &ADF4106_Data[13].chData;
volatile char* 	adf4106_CPZ			= &ADF4106_Data[14].chData;
volatile char* 	adf4106_PDPol		= &ADF4106_Data[15].chData;
volatile char* 	adf4106_Mux			= &ADF4106_Data[16].chData;
volatile char* 	adf4106_PD1			= &ADF4106_Data[17].chData;
volatile char* 	adf4106_CntRst		= &ADF4106_Data[18].chData;
volatile char* 	WriteRef 			= &ADF4106_Data[19].chData;
volatile char* 	WriteN				= &ADF4106_Data[20].chData;
volatile char* 	WriteFunc 			= &ADF4106_Data[21].chData;
volatile char* 	WriteInitLatch 		= &ADF4106_Data[22].chData;

static char prevWriteRef = 0;
static char prevWriteN	= 0;
static char prevWriteFunc = 0;
static char prevWriteInitLatch = 0;

/**************************************************************************//**
 * @brief Initialize SPI and Initial Values for ADF4106 Board.
 *
 * @param None.
 *
 * @return TRUE or FALSE.
******************************************************************************/
int ADF4106_Init( void )
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};

	// setup SPI interface
	SetCtrl(ADF4106_SPI_CFG);

    // Set GPIO0(LE) and GPIO2(CE) as outputs
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x05);

    // Set GPIO0(LE) and GPIO2(CE) high
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, 0x05);

    // Initialization Latch Method (datasheet pg. 18)
    // 1. Apply VDD
    // 2. Program Initialization Latch, F1 programmed 0
    	/*
    	 * Prescaler Value = 16/17
    	 * Timeout (PFD Cycles) = 3
    	 * FASTLOCK Disabled
    	 * Charge Pump Output Normal
    	 * MUXOUT = R Divider Output
    	 * Counter Operation Normal
    	 */
		cfgValue = 0x5F8083;

		// Arrange data in 8 bit packets
		txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
		txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
		txBuffer[2] = cfgValue & 0x0000FF;

		// LE = 0
		IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x04);

		// Transfer data via SPI
		alt_avalon_spi_command(ADF4106_SPI, 0, 3, txBuffer, 0, NULL, 0);

		// LE = 1
		IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x05);
	// 3. Do a Function Latch load, F1 programmed 0
    	/*
    	 * Prescaler Value = 16/17
    	 * Timeout (PFD Cycles) = 3
    	 * FASTLOCK Disabled
    	 * Charge Pump Output Normal
    	 * MUXOUT = R Divider Output
    	 * Counter Operation Normal
    	 */
		cfgValue = 0x5F8082;

		// Arrange data in 8 bit packets
		txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
		txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
		txBuffer[2] = cfgValue & 0x0000FF;

		// LE = 0
		IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x04);

		// Transfer data via SPI
		alt_avalon_spi_command(ADF4106_SPI, 0, 3, txBuffer, 0, NULL, 0);

		// LE = 1
		IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x05);
	// 4. Do a R Counter Latch load
		/*
		 * R = 10
		 */
		cfgValue = 0x000028;

		// Arrange data in 8 bit packets
		txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
		txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
		txBuffer[2] = cfgValue & 0x0000FF;

		// LE = 0
		IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x04);

		// Transfer data via SPI
		alt_avalon_spi_command(ADF4106_SPI, 0, 3, txBuffer, 0, NULL, 0);

		// LE = 1
		IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x05);
	// 5. Do a N(A, B) Counter Latch load
		/*
		 * B = 362
		 * A = 8
		 */
		cfgValue = 0x016A21;

		// Arrange data in 8 bit packets
		txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
		txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
		txBuffer[2] = cfgValue & 0x0000FF;

		// LE = 0
		IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x04);

		// Transfer data via SPI
		alt_avalon_spi_command(ADF4106_SPI, 0, 3, txBuffer, 0, NULL, 0);

		// LE = 1
		IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x05);
	// Final Result: (B   * P  + A) * PFD(kHz) = RFout (MHz)
	//				 (362 * 16 + 8) * 1000     = 5800
	return(SUCCESS);
}

void ADF4106_WriteRefCounter(void)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select command type
	cfgValue = (*adf4106_LDPbtn << ADF4106_LDP)  	|
               (*adf4106_ABP2btn << ADF4106_ABP2) 	| 
			   (*adf4106_ABP1btn << ADF4106_ABP1)  	| 
			   (*adf4106_RSlider << ADF4106_RValue)	| 
			   (ADF4106_RCounter);

	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = cfgValue & 0x0000FF;

	// LE = 0
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x04);

	// Transfer data via SPI
	alt_avalon_spi_command(ADF4106_SPI, 0, 3, txBuffer, 0, NULL, 0);

	// LE = 1
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x05);
}

void ADF4106_WriteNCounter(void)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select command type
	cfgValue = (*adf4106_CPGainbtn << ADF4106_CPGain)   |
               (*adf4106_BSlider << ADF4106_BValue) 	| 
			   (*adf4106_ASlider << ADF4106_AValue)  	| 
			   (ADF4106_NCounter);

	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = cfgValue & 0x0000FF;
	
	// LE = 0
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x04);

	// Transfer data via SPI
	alt_avalon_spi_command(ADF4106_SPI, 0, 3, txBuffer, 0, NULL, 0);

	// LE = 1
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x05);
}

void ADF4106_WriteFuncLatch(void)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select command type
	cfgValue = (*adf4106_PrescVal << ADF4106_Prescaler) 		|
			   (*adf4106_PD2 << ADF4106_PowerDown2) 			|
			   (*adf4106_CurrSet2 << ADF4106_CurrentSetting2)	|
			   (*adf4106_CurrSet1 << ADF4106_CurrentSetting1) 	|
			   (*adf4106_TimerCtrl << ADF4106_TimerControl) 	|
			   (*adf4106_FastLock << ADF4106_FastlockMode) 		|
			   (*adf4106_FastEn << ADF4106_FastlockEn) 			|
			   (*adf4106_CPZ << ADF4106_CPThreeState) 			|
			   (*adf4106_PDPol << ADF4106_PDPolarity) 			|
			   (*adf4106_Mux << ADF4106_Muxout) 				|
			   (*adf4106_PD1 << ADF4106_PowerDown1) 			|
			   (*adf4106_CntRst << ADF4106_CounterRst) 			|
			   (ADF4106_Function);

	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = cfgValue & 0x0000FF;
	
	// LE = 0
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x04);

	// Transfer data via SPI
	alt_avalon_spi_command(ADF4106_SPI, 0, 3, txBuffer, 0, NULL, 0);

	// LE = 1
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x05);
}

void ADF4106_WriteInitLatch(void)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select command type
	cfgValue = (*adf4106_PrescVal << ADF4106_Prescaler) 		|
			   (*adf4106_PD2 << ADF4106_PowerDown2) 			|
			   (*adf4106_CurrSet2 << ADF4106_CurrentSetting2)	|
			   (*adf4106_CurrSet1 << ADF4106_CurrentSetting1) 	|
			   (*adf4106_TimerCtrl << ADF4106_TimerControl) 	|
			   (*adf4106_FastLock << ADF4106_FastlockMode) 		|
			   (*adf4106_FastEn << ADF4106_FastlockEn) 			|
			   (*adf4106_CPZ << ADF4106_CPThreeState) 			|
			   (*adf4106_PDPol << ADF4106_PDPolarity) 			|
			   (*adf4106_Mux << ADF4106_Muxout) 				|
			   (*adf4106_PD1 << ADF4106_PowerDown1) 			|
			   (*adf4106_CntRst << ADF4106_CounterRst) 			|
			   (ADF4106_cmdInit);

	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = cfgValue & 0x0000FF;
	
	// LE = 0
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x04);

	// Transfer data via SPI
	alt_avalon_spi_command(ADF4106_SPI, 0, 3, txBuffer, 0, NULL, 0);

	// LE = 1
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x05);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return TRUE
******************************************************************************/
int ADF4106_UpdateParameters(void)
{
	if(*WriteRef != prevWriteRef)
	{
		ADF4106_WriteRefCounter();
		prevWriteRef = *WriteRef;
	}
	if(*WriteN != prevWriteN)
	{
		ADF4106_WriteNCounter();
		prevWriteN = *WriteN;
	}
	if(*WriteFunc != prevWriteFunc)
	{
		ADF4106_WriteFuncLatch();
		prevWriteFunc = *WriteFunc;
	}
	if(*WriteInitLatch != prevWriteInitLatch)
	{
		ADF4106_WriteInitLatch();
		prevWriteInitLatch = *WriteInitLatch;
	}
    return(SUCCESS);
}
#endif
